from abc import ABC, abstractmethod
import re
from tablib import Dataset


class Database(ABC):
    """A base database client class with default DB API 2 implementations
    """
    def __init__(self) -> None:
        super().__init__()
        self.connection = None
        self.database_url: DatabaseURL = None
        self.read_keywords = ['select']
        self.write_keywords = ['insert', 'update', 'delete', 'merge']

    @abstractmethod
    def connect(self, database_url: str):
        """Create a connection to the database.

        The database url has this form as SQLAlchemy does:
        dialect+driver://username:password@host:port/database

        Check the following link for examples:
        https://docs.sqlalchemy.org/en/latest/core/engines.html

        :param database_url: the database url
        :return: None, should set up the instance variable self.connection though
        """
        self.database_url = DatabaseURL(database_url)
        self.connection = None

    def read(self, query: str, limit: int = None) -> Dataset:
        """Run a read(select) query to the database.

        It should return a list of tuples,
        the first tuple is metadata, each element is a tuple of column name and data type,
        the rest are the records.

        :param query: the sql query
        :param limit: the maximum number of records to return
        :return: a list of tuples
        """
        data = Dataset()
        cursor = self.connection.cursor()
        query = self.add_row_limit_in_query(query, limit)
        cursor.execute(query)
        result = cursor.fetchall()

        if cursor.description:
            data.headers = ['{0}({1})'.format(d[0], self.get_data_type(d[1])) for d in cursor.description]
        data.extend(result)
        cursor.close()

        return data

    @abstractmethod
    def add_row_limit_in_query(self, query, limit):
        """This adds a filter into the query to limit the number of rows.
        It depends on the specific database SQL.
        This is for performance consideration that fetchmany does not work well with large data set.

        Such as to add a limit 10 or top 10 into the query if it does not have it.
        """
        pass

    def write(self, query: str) -> int:
        """Run a write(insert, update, delete) query to the database.

        :param query: the sql query
        :return: number of rows affected
        """
        cursor = self.connection.cursor()
        cursor.execute(query)
        return cursor.rowcount

    def query(self, query: str, limit: int = None):
        """
        Dispatch to read or write accordingly.
        Sub classes should wrapper it around to transfer errors to TRecordError.

        :param query: the sql query
        :param limit: the maximum number of records to return
        :return: what read or write should return
        """
        first_key_word = re.split(r'\s+', query.strip().lower(), 1)[0]
        if first_key_word in self.read_keywords:
            return self.read(query, limit)
        elif first_key_word in self.write_keywords:
            return self.write(query)
        else:
            return self.read(query)

    @abstractmethod
    def get_version(self) -> str:
        pass

    @abstractmethod
    def get_current_db(self) -> str:
        pass

    @abstractmethod
    def get_tables(self, database=None) -> list:
        pass

    @abstractmethod
    def get_ddl(self, table, database=None):
        pass

    @abstractmethod
    def get_data_type(self, type_code: int) -> str:
        """
        Check the type code and return a string description
        :param type_code:
        :return:
        """
        pass

    def close(self):
        self.connection.close()


class DatabaseURL:
    """This class is mainly to extract database url into individual elements"""
    def __init__(self, database_url: str):
        self.database_url = database_url
        self.dialect = None
        self.driver = None
        self.username = None
        self.password = None
        self.host = None
        self.port = None
        self.database = None
        self.extract_info()

    def extract_info(self):
        """
        Extract elements from database url: dialect+driver://username:password@host:port/database
        :return:
        """
        pattern = re.compile(r'([a-z]+)\+?([a-z]+)?://([a-zA-Z0-9_]+):(.+?)@([a-zA-Z0-9._]+):?([0-9]+)?/?([a-zA-Z0-9_]+)?')
        m = pattern.match(self.database_url)
        if m:
            self.dialect, self.driver, self.username, self.password, self.host, self.port, self.database = m.groups()
        else:
            raise RuntimeError('The database url is invalid.')

    def __str__(self) -> str:
        return self.database_url


if __name__ == '__main__':
    u = DatabaseURL('mysql://scott:tiger~*&%@$_+@localhost/foo')
    print(u.dialect, u.driver, u.username, u.password, u.host, u.port, u.database)



from trecord.database import Database
from trecord.error import TRecordError
import pyodbc
import re
from datetime import datetime, date


class PyODBCMSSQL(Database):
    """This class implements the database using PyMySQL"""

    def __init__(self) -> None:
        super().__init__()

    def connect(self, database_url: str):
        super().connect(database_url)
        self.connection = pyodbc.connect(
                f'DSN={self.database_url.host};UID={self.database_url.username};PWD={self.database_url.password}'
            )
        if self.database_url.database:
            self.write("USE {}".format(self.database_url.database))

    def query(self, query: str, limit: int = None):
        try:
            result = super().query(query, limit)
            return result
        except pyodbc.Error as err:
            raise TRecordError(err)

    def add_row_limit_in_query(self, query, limit):
        if limit and not re.search(r'\s+top\s+', query, re.IGNORECASE):
            query = re.sub(r'^select\s+', 'select top {} '.format(limit), query)
        return query

    def get_data_type(self, type_code: int) -> str:
        """
        Check the type code and return a string description
        :param type_code:
        :return:
        """
        if type_code is int:
            return 'INTEGER'
        elif type_code is bool:
            return 'BOOLEAN'
        elif type_code is datetime:
            return 'DATETIME'
        elif type_code is date:
            return 'DATE'

        mapping = {
            pyodbc.STRING: 'STRING',
            pyodbc.BINARY: 'BINARY',
            pyodbc.NUMBER: 'NUMBER',
            pyodbc.Date: 'DATE',
            pyodbc.Time: 'TIME',
            pyodbc.Timestamp: 'TIMESTAMP'
        }
        for key in mapping.keys():
            if type_code is key:
                return mapping[key]
            else:
                return str(type_code)

    def get_version(self) -> str:
        return self.query('SELECT @@VERSION')[0][0]

    def get_current_db(self) -> str:
        return self.query('SELECT DB_NAME()')[0][0]

    def get_tables(self, database=None) -> list:
        if not database:
            database = self.get_current_db()
        return list(
            self.query("select table_name "
                       "from {}.information_schema.tables;".format(database)).get_col(0)
        )

    def get_ddl(self, table, database=None):
        if not database:
            database = self.get_current_db()
        ddl = ''
        ddl += self.query(
            "select table_catalog, table_schema, table_name, column_name, data_type, character_maximum_length, "
            "numeric_precision, numeric_scale from {}.INFORMATION_SCHEMA.COLUMNS "
            "where table_name = '{}' order by ordinal_position;".format(database, table)).__str__()

        ddl += '\n\n'

        constraints_query = """SELECT
            constraint_name, ordinal_position, table_name, column_name, is_unique, is_primary_key
            FROM {}.INFORMATION_SCHEMA.KEY_COLUMN_USAGE k
            JOIN SYS.INDEXES i
            ON   k.constraint_name = i.name
            WHERE table_name = '{}'
            ORDER BY constraint_name, ordinal_position
            """.format(database, table)
        ddl += self.query(constraints_query).__str__()

        return ddl


if __name__ == '__main__':
    import sys

    sql = PyODBCMSSQL()
    sql.connect(sys.argv[1])
    print(sql.get_version())
    print(sql.get_current_db())
    print(sql.get_ddl('vw_eBay_NiceLinksForSurvey'))
    print(sql.get_ddl('tblREScorecardRule'))



from trecord.database import Database
import cx_Oracle


class CxOracle(Database):
    """This class implements the database using cx_Oracle"""
    def __init__(self) -> None:
        super().__init__()

    def connect(self, database_url: str):
        pass

    def get_data_type(self, type_code: int) -> str:
        pass




COMMAND_KEYWORDS = ['help', 'limit', 'tables', 'ddl', 'quit']

SQL_KEYWORDS = ['ABS',
                'ABSOLUTE',
                'ACOS',
                'ACTION',
                'ADD',
                'AFTER',
                'ALL',
                'ALLOCATE',
                'ALTER',
                'AND',
                'ANY',
                'ARE',
                'ARRAY',
                'ARRAY_AGG',
                'ARRAY_MAX_CARDINALITY',
                'AS',
                'ASC',
                'ASENSITIVE',
                'ASIN',
                'ASSERTION',
                'ASYMMETRIC',
                'AT',
                'ATAN',
                'ATOMIC',
                'AUTHORIZATION',
                'AVG',
                'BEFORE',
                'BEGIN',
                'BEGIN_FRAME',
                'BEGIN_PARTITION',
                'BETWEEN',
                'BIGINT',
                'BINARY',
                'BIT',
                'BIT_LENGTH',
                'BLOB',
                'BOOLEAN',
                'BOTH',
                'BREADTH',
                'BY',
                'CALL',
                'CALLED',
                'CARDINALITY',
                'CASCADE',
                'CASCADED',
                'CASE',
                'CAST',
                'CATALOG',
                'CEIL',
                'CEILING',
                'CHAR',
                'CHAR_LENGTH',
                'CHARACTER',
                'CHARACTER_LENGTH',
                'CHECK',
                'CLASSIFIER',
                'CLOB',
                'CLOSE',
                'COALESCE',
                'COLLATE',
                'COLLATION',
                'COLLECT',
                'COLUMN',
                'COMMIT',
                'CONDITION',
                'CONNECT',
                'CONNECTION',
                'CONSTRAINT',
                'CONSTRAINTS',
                'CONSTRUCTOR',
                'CONTAINS',
                'CONTINUE',
                'CONVERT',
                'COPY',
                'CORR',
                'CORRESPONDING',
                'COS',
                'COSH',
                'COUNT',
                'COVAR_POP',
                'COVAR_SAMP',
                'CREATE',
                'CROSS',
                'CUBE',
                'CUME_DIST',
                'CURRENT',
                'CURRENT_CATALOG',
                'CURRENT_DATE',
                'CURRENT_DEFAULT_TRANSFORM_GROUP',
                'CURRENT_PATH',
                'CURRENT_ROLE',
                'CURRENT_ROW',
                'CURRENT_SCHEMA',
                'CURRENT_TIME',
                'CURRENT_TIMESTAMP',
                'CURRENT_TRANSFORM_GROUP_FOR_TYPE',
                'CURRENT_USER',
                'CURSOR',
                'CYCLE',
                'DATA',
                'DATE',
                'DAY',
                'DEALLOCATE',
                'DEC',
                'DECFLOAT',
                'DECIMAL',
                'DECLARE',
                'DEFAULT',
                'DEFERRABLE',
                'DEFERRED',
                'DEFINE',
                'DELETE',
                'DENSE_RANK',
                'DEPTH',
                'DEREF',
                'DESC',
                'DESCRIBE',
                'DESCRIPTOR',
                'DETERMINISTIC',
                'DIAGNOSTICS',
                'DISCONNECT',
                'DISTINCT',
                'DO',
                'DOMAIN',
                'DOUBLE',
                'DROP',
                'DYNAMIC',
                'EACH',
                'ELEMENT',
                'ELSE',
                'ELSEIF',
                'EMPTY',
                'END',
                'END_FRAME',
                'END_PARTITION',
                'END-EXEC',
                'EQUALS',
                'ESCAPE',
                'EVERY',
                'EXCEPT',
                'EXCEPTION',
                'EXEC',
                'EXECUTE',
                'EXISTS',
                'EXIT',
                'EXP',
                'EXTERNAL',
                'EXTRACT',
                'FALSE',
                'FETCH',
                'FILTER',
                'FIRST',
                'FIRST_VALUE',
                'FLOAT',
                'FOR',
                'FOREIGN',
                'FOUND',
                'FRAME_ROW',
                'FREE',
                'FROM',
                'FULL',
                'FUNCTION',
                'FUSION',
                'GENERAL',
                'GET',
                'GLOBAL',
                'GO',
                'GOTO',
                'GRANT',
                'GROUP',
                'GROUPING',
                'GROUPS',
                'HANDLER',
                'HAVING',
                'HOLD',
                'HOUR',
                'IDENTITY',
                'IF',
                'IMMEDIATE',
                'IN',
                'INDICATOR',
                'INITIAL',
                'INITIALLY',
                'INNER',
                'INOUT',
                'INPUT',
                'INSENSITIVE',
                'INSERT',
                'INT',
                'INTEGER',
                'INTERSECT',
                'INTERSECTION',
                'INTERVAL',
                'INTO',
                'IS',
                'ISOLATION',
                'ITERATE',
                'JOIN',
                'JSON_ARRAY',
                'JSON_ARRAYAGG',
                'JSON_EXISTS',
                'JSON_OBJECT',
                'JSON_OBJECTAGG',
                'JSON_QUERY',
                'JSON_TABLE',
                'JSON_TABLE_PRIMITIVE',
                'JSON_VALUE',
                'KEY',
                'LAG',
                'LANGUAGE',
                'LARGE',
                'LAST',
                'LAST_VALUE',
                'LATERAL',
                'LEAD',
                'LEADING',
                'LEAVE',
                'LEFT',
                'LEVEL',
                'LIKE',
                'LIKE_REGEX',
                'LIMIT',
                'LISTAGG',
                'LN',
                'LOCAL',
                'LOCALTIME',
                'LOCALTIMESTAMP',
                'LOCATOR',
                'LOG',
                'LOG10',
                'LOOP',
                'LOWER',
                'MAP',
                'MATCH',
                'MATCH_NUMBER',
                'MATCH_RECOGNIZE',
                'MATCHES',
                'MAX',
                'MEMBER',
                'MERGE',
                'METHOD',
                'MIN',
                'MINUTE',
                'MOD',
                'MODIFIES',
                'MODULE',
                'MONTH',
                'MULTISET',
                'NAMES',
                'NATIONAL',
                'NATURAL',
                'NCHAR',
                'NCLOB',
                'NEW',
                'NEXT',
                'NO',
                'NONE',
                'NORMALIZE',
                'NOT',
                'NTH_VALUE',
                'NTILE',
                'NULL',
                'NULLIF',
                'NUMERIC',
                'OBJECT',
                'OCCURRENCES_REGEX',
                'OCTET_LENGTH',
                'OF',
                'OFFSET',
                'OLD',
                'OMIT',
                'ON',
                'ONE',
                'ONLY',
                'OPEN',
                'OPTION',
                'OR',
                'ORDER',
                'ORDINALITY',
                'OUT',
                'OUTER',
                'OUTPUT',
                'OVER',
                'OVERLAPS',
                'OVERLAY',
                'PAD',
                'PARAMETER',
                'PARTIAL',
                'PARTITION',
                'PATH',
                'PATTERN',
                'PER',
                'PERCENT',
                'PERCENT_RANK',
                'PERCENTILE_CONT',
                'PERCENTILE_DISC',
                'PERIOD',
                'PORTION',
                'POSITION',
                'POSITION_REGEX',
                'POWER',
                'PRECEDES',
                'PRECISION',
                'PREPARE',
                'PRESERVE',
                'PRIMARY',
                'PRIOR',
                'PRIVILEGES',
                'PROCEDURE',
                'PTF',
                'PUBLIC',
                'RANGE',
                'RANK',
                'READ',
                'READS',
                'REAL',
                'RECURSIVE',
                'REF',
                'REFERENCES',
                'REFERENCING',
                'REGR_AVGX',
                'REGR_AVGY',
                'REGR_COUNT',
                'REGR_INTERCEPT',
                'REGR_R2',
                'REGR_SLOPE',
                'REGR_SXX',
                'REGR_SXY',
                'REGR_SYY',
                'RELATIVE',
                'RELEASE',
                'REPEAT',
                'RESIGNAL',
                'RESTRICT',
                'RESULT',
                'RETURN',
                'RETURNS',
                'REVOKE',
                'RIGHT',
                'ROLE',
                'ROLLBACK',
                'ROLLUP',
                'ROUTINE',
                'ROW',
                'ROW_NUMBER',
                'ROWS',
                'RUNNING',
                'SAVEPOINT',
                'SCHEMA',
                'SCOPE',
                'SCROLL',
                'SEARCH',
                'SECOND',
                'SECTION',
                'SEEK',
                'SELECT',
                'SENSITIVE',
                'SESSION',
                'SESSION_USER',
                'SET',
                'SETS',
                'SHOW',
                'SIGNAL',
                'SIMILAR',
                'SIN',
                'SINH',
                'SIZE',
                'SKIP',
                'SMALLINT',
                'SOME',
                'SPACE',
                'SPECIFIC',
                'SPECIFICTYPE',
                'SQL',
                'SQLCODE',
                'SQLERROR',
                'SQLEXCEPTION',
                'SQLSTATE',
                'SQLWARNING',
                'SQRT',
                'START',
                'STATE',
                'STATIC',
                'STDDEV_POP',
                'STDDEV_SAMP',
                'SUBMULTISET',
                'SUBSET',
                'SUBSTRING',
                'SUBSTRING_REGEX',
                'SUCCEEDS',
                'SUM',
                'SYMMETRIC',
                'SYSTEM',
                'SYSTEM_TIME',
                'SYSTEM_USER',
                'TABLE',
                'TABLESAMPLE',
                'TAN',
                'TANH',
                'TEMPORARY',
                'THEN',
                'TIME',
                'TIMESTAMP',
                'TIMEZONE_HOUR',
                'TIMEZONE_MINUTE',
                'TO',
                'TRAILING',
                'TRANSACTION',
                'TRANSLATE',
                'TRANSLATE_REGEX',
                'TRANSLATION',
                'TREAT',
                'TRIGGER',
                'TRIM',
                'TRIM_ARRAY',
                'TRUE',
                'TRUNCATE',
                'UESCAPE',
                'UNDER',
                'UNDO',
                'UNION',
                'UNIQUE',
                'UNKNOWN',
                'UNNEST',
                'UNTIL',
                'UPDATE',
                'UPPER',
                'USAGE',
                'USER',
                'USING',
                'VALUE',
                'VALUES',
                'VALUE_OF',
                'VAR_POP',
                'VAR_SAMP',
                'VARBINARY',
                'VARCHAR',
                'VARYING',
                'VERSIONING',
                'VIEW',
                'WHEN',
                'WHENEVER',
                'WHERE',
                'WHILE',
                'WIDTH_BUCKET',
                'WINDOW',
                'WITH',
                'WITHIN',
                'WITHOUT',
                'WORK',
                'WRITE',
                'YEAR',
                'ZONE',
                ]



from trecord.mssql import PyODBCMSSQL
from trecord.mysql import PyMySQL
from trecord.oracle import CxOracle
from trecord.database import DatabaseURL, Database
from trecord.error import TRecordError
from trecord.command import Command


def get_database_by_url(url: str):
    db_url = DatabaseURL(url)
    if db_url.dialect == 'mssql':
        db = PyODBCMSSQL()
    elif db_url.dialect == 'mysql':
        db = PyMySQL()
    elif db_url.dialect == 'oracle':
        db = CxOracle()
    else:
        raise NotImplementedError('The database is not implemented yet.')

    db.connect(url)
    return db



from trecord.database import Database
from trecord.error import TRecordError
from trecord.version import VERSION
from trecord.mysql import PyMySQL
from prompt_toolkit import PromptSession, print_formatted_text, HTML
from prompt_toolkit.styles import Style, style_from_pygments_cls, merge_styles
from prompt_toolkit.lexers import PygmentsLexer
from pygments.lexers.sql import TransactSqlLexer
from pygments.styles.rainbow_dash import RainbowDashStyle
import sys
from tabulate import tabulate
from tablib import Dataset
import re
from trecord.keywords import COMMAND_KEYWORDS, SQL_KEYWORDS
from prompt_toolkit.completion import WordCompleter


class Command:
    """This class is an interactive command line client with Database"""
    def __init__(self, database: Database):
        self.database = database
        self.limit = 10
        self.message = None
        self.style = None
        self.setup_prompt()
        self.keywords = COMMAND_KEYWORDS + SQL_KEYWORDS + [k.lower() for k in SQL_KEYWORDS]
        self.session = PromptSession(style=self.style, lexer=PygmentsLexer(TransactSqlLexer),
                                     completer=WordCompleter(self.keywords), complete_while_typing=True)
        self.welcome()

    def welcome(self):
        print('Database {}'.format(self.database.get_version()))
        print('TRecord {}. Type "?" or "help" for help.'.format(VERSION))
        print()

    def set_limit(self, limit):
        self.limit = limit

    def setup_prompt(self):
        self.style = merge_styles([
            style_from_pygments_cls(RainbowDashStyle),

            Style.from_dict({
                'username': '#8dc3fc',
                'punctuation': '#090908',
                'host': '#8dc3fc',
                'database': '#aa83fc'
            })
        ])

        self.message = [
            ('class:username', self.database.database_url.username),
            ('class:punctuation', '@'),
            ('class:host', self.database.database_url.host),
            ('class:punctuation', ':'),
            ('class:database', self.database.get_current_db()),
            ('class:punctuation', '> '),
        ]

    @staticmethod
    def exit():
        print_formatted_text(HTML('<b>Quit.</b>'))
        sys.exit()

    def run_query_to_output(self, query):
        dataset = self.database.query(query, limit=self.limit)
        if dataset.height:
            print(tabulate(dataset, headers=dataset.headers, tablefmt='psql'))
            print('\n{} rows returned/limited'.format(dataset.height))
        else:
            if query.lower().startswith('select'):
                print('no rows returned')
            else:
                print()

    @staticmethod
    def print_error(content):
        print_formatted_text(HTML('<red>{}</red>'.format(content)))

    @staticmethod
    def print_message(content):
        print_formatted_text(HTML('<green>{}</green>'.format(content)))

    def run_command(self, command: str):
        if command == '.quit':
            self.exit()
        elif command.startswith('.limit'):
            try:
                self.limit = int(re.split(r'\s+', command)[1])
            except (ValueError, IndexError):
                self.print_error('limit is required and needs to be an integer.')
        elif command.startswith('.tables'):
            database = None
            try:
                database = re.split(r'\s+', command)[1]
            except IndexError:
                pass

            for table in self.database.get_tables(database):
                print(table)
            print()
        elif command.startswith('.ddl'):
            try:
                arg = re.split(r'\s+', command)[1]
                table = arg
                database = None
                if '.' in arg:
                    database, table = arg.split('.')
                print(self.database.get_ddl(table, database))
                print()
            except IndexError:
                self.print_error('Table is required.')
        else:
            self.print_error('Command {} is not recognized'.format(command))

    @staticmethod
    def help():
        helps = Dataset()
        helps.headers = ['Input Form', 'Description']
        helps.append(['?', 'Print this help document.'])
        helps.append(['help', 'Print this help document.'])
        helps.append(['.limit <INTEGER>', 'Set the query limit.'])
        helps.append(['.tables [DATABASE]',
                      'Fetch the list of tables in current database or from the specified database.'])
        helps.append(['.ddl <[DATABASE.]TABLE>',
                      'Fetch the DDL of the table in current database or other if it is fully qualified.'])
        helps.append(['.quit', 'Quit.'])
        helps.append(['<QUERY>', 'Any SQL query.'])
        print(tabulate(helps, tablefmt='fancy_grid'))

    def loop(self):
        query_lines = []
        while True:
            try:
                this_line = self.session.prompt(self.message)
                this_line = this_line.strip()

                if this_line in ['help', '?']:
                    self.help()
                elif this_line.startswith('.'):
                    self.run_command(this_line)
                else:
                    query_lines.append(this_line)
                    if this_line.endswith(';'):
                        query = '\n'.join(query_lines).strip()
                        self.run_query_to_output(query)
                        query_lines = []

                        if query.lower().startswith('use'):  # reset the database at the prompt
                            self.setup_prompt()
            except TRecordError as err:
                query_lines = []
                self.print_error(err)
            except KeyboardInterrupt:
                query_lines = []
                self.print_message('Query input reset.')
            except EOFError:
                self.exit()


if __name__ == '__main__':
    # 'mysql+pymysql://lusaisai:lusaisai@198.58.115.91/employees'
    db = PyMySQL()
    db.connect(sys.argv[1])
    cmd = Command(db)
    cmd.loop()




from trecord.database import Database
from trecord.error import TRecordError
import pymysql
import re


class PyMySQL(Database):
    """This class implements the database using PyMySQL"""
    def __init__(self) -> None:
        super().__init__()

    def connect(self, database_url: str):
        super().connect(database_url)
        self.connection = pymysql.connect(host=self.database_url.host,
                                          user=self.database_url.username,
                                          password=self.database_url.password,
                                          db=self.database_url.database,
                                          charset='utf8mb4')

    def query(self, query: str, limit: int = None):
        try:
            result = super().query(query, limit)
            return result
        except pymysql.Error as err:
            raise TRecordError(err)

    def add_row_limit_in_query(self, query, limit):
        if limit and not re.search(r'\s+limit\s+', query, re.IGNORECASE):
            query = query.strip(' ;') + ' limit {};'.format(limit)
        return query

    def get_data_type(self, type_code: int) -> str:
        """
        Check the type code and return a string description
        :param type_code:
        :return:
        """
        mapping = {
            pymysql.STRING: 'STRING',
            pymysql.BINARY: 'BINARY',
            pymysql.NUMBER: 'NUMBER',
            pymysql.DATE: 'DATE',
            pymysql.TIME: 'TIME',
            pymysql.TIMESTAMP: 'TIMESTAMP',
            pymysql.ROWID: 'ROWID'
        }
        for key in mapping.keys():
            if type_code == key:
                return mapping[key]

    def get_version(self) -> str:
        return self.query('select version()')[0][0]

    def get_current_db(self) -> str:
        db = self.query('select database()')[0][0]
        if not db:
            db = ''
        return db

    def get_tables(self, database=None) -> list:
        if database:
            return list(
                self.query("select table_name "
                           "from information_schema.tables "
                           "where TABLE_SCHEMA='{}';".format(database)).get_col(0)
            )
        else:
            return list(self.query('show tables;').get_col(0))

    def get_ddl(self, table, database=None):
        if not database:
            database = self.get_current_db()
        return self.query('show create table {}.{};'.format(database, table))[0][1]


if __name__ == '__main__':
    sql = PyMySQL()
    sql.connect('mysql+pymysql://lusaisai:lusaisai@198.58.115.91/employees')
    print(sql.get_version())
    print(sql.get_current_db())
    print(sql.get_tables())
    print(sql.get_ddl('employees'))
    print(sql.query('select * from departments;', 5))
    print(sql.query('select * from employees;', 20))



from setuptools import setup

setup(
    name='TRecord',
    version='1.0.0',
    packages=['trecord'],
    url='https://github.com/lusaisai/TRecord',
    license='MIT',
    author='lusaisai',
    author_email='lusaisai@163.com',
    description='An Interactive SQL Client',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pygments',
        'PyMySQL',
        'pyodbc',
        'cx_Oracle',
        'prompt_toolkit',
        'tabulate[widechars]',
        'tablib',
        'click'
    ],
    scripts=[
        'scripts/trecord'
    ]
)
